{***************************************************************************}
{ TDBFinder freeware components by Deborah Pate                             }
{ Version 1.81, May 1999                                                    }
{                                                                           }
{ Registers TDBFinder controls, data-aware controls for                     }
{ finding records in 32-bit Delphi.                                         }
{                                                                           }
{ The TDBFinder controls in this version are TDBFinderEdit & TDBFinderCombo.}
{                                                                           }
{ Please send comments, bug reports, suggestions to:                        }
{ Debs@djpate.freeserve.co.uk                                               }	
{***************************************************************************}
unit DBFinder;

interface

uses
  Classes, DsgnIntf;

type
  TDBFDisplayFieldProperty = class(TStringProperty)
{ Property editor for the DisplayField property, needed because the DataField
  property editor is (annoyingly) buried in the implementation section of the
  DBReg unit. }
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    function GetDataSourcePropName: string; virtual;
    procedure GetValueList(var List: TStrings); virtual;
  end;

procedure Register;


implementation

uses
   TypInfo, DB, DBFCombo, DBFEdit;


{ TDBFDisplayFieldProperty }

function TDBFDisplayFieldProperty.GetAttributes: TPropertyAttributes;
begin
   Result := inherited GetAttributes + [paValuelist, paSortList] - [paMultiSelect];
end;

procedure TDBFDisplayFieldProperty.GetValues(Proc: TGetStrProc);
var
  i: integer;
  FieldNames: TStrings;
begin
  FieldNames := TStringList.Create;
  try
    { GetValueList retrieves field names from the dataset }
    GetValueList(FieldNames);
    { Then we feed them to Proc in the usual way }
    for i := 0 to FieldNames.Count - 1 do Proc(FieldNames[i]);
  finally
    FieldNames.Free;
    FieldNames := nil;
  end;
end;

function TDBFDisplayFieldProperty.GetDataSourcePropName: string;
begin
  Result := 'DataSource';
end;

procedure TDBFDisplayFieldProperty.GetValueList(var List: TStrings);
{ Here's where we retrieve the field names from the dataset }
var
  Component: TComponent;
  PropertyInfo: PPropInfo;
  DataSource: TDataSource;
begin
  Component := (GetComponent(0) as TComponent);
  PropertyInfo := TypInfo.GetPropInfo(Component.ClassInfo, GetDataSourcePropName);
  if (PropertyInfo <> nil) and (PropertyInfo^.PropType^.Kind = tkClass) then
  begin
    DataSource := TObject(GetOrdProp(Component, PropertyInfo)) as TDataSource;
    if (DataSource <> nil) and (DataSource.DataSet <> nil) then
      DataSource.DataSet.GetFieldNames(List);
  end;
end;

{ Register the components }
procedure Register;
begin
  RegisterComponents('Data Controls', [TDBFinderCombo, TDBFinderEdit]);
  RegisterPropertyEditor(TypeInfo(string),TDBFinderCombo,'DisplayField',TDBFDisplayFieldProperty);
end;

end.
